import { Button } from '@material-ui/core'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import React, { useCallback, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import TermsService from 'services/terms.service'
import { updateTermsStatus } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import CGUModal from './CGUModal'
import DataShareConsentContent from './DataShareConsentContent'
import LegalNoticeModal from './LegalNoticeModal'
import MinorUpdateContent from './MinorUpdateContent'
import './termsView.scss'

const TermsView = () => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const dispatch = useAppDispatch()
  const { termsStatus } = useAppSelector(state => state.ecolyo.global)
  const [GCUValidation, setGCUValidation] = useState(false)
  const [dataConsentValidation, setDataConsentValidation] = useState(false)
  const [openCGUModal, setOpenCGUModal] = useState(false)
  const [openLegalNoticeModal, setOpenLegalNoticeModal] = useState(false)

  const handleTermValidate = useCallback(async () => {
    const termsService = new TermsService(client)
    const createdTerm = await termsService.createTerm()
    if (createdTerm) {
      dispatch(
        updateTermsStatus({
          accepted: true,
          versionType: await termsService.getTermsVersionType(),
        })
      )
    }
    navigate('/consumption')
  }, [dispatch, client, navigate])

  return (
    <div className="terms-wrapper">
      {termsStatus.versionType === 'major' ||
      termsStatus.versionType === 'init' ? (
        <>
          <div className="terms-content">
            <DataShareConsentContent />
            <label htmlFor="dataConsent" className="inline">
              <input
                id="dataConsent"
                type="checkbox"
                className="inputCheckbox"
                onChange={e => setDataConsentValidation(e.target.checked)}
                checked={dataConsentValidation}
                required
              />
              {t('dataShare.validDataConsent')}
            </label>
            <label htmlFor="gcu" className="inline">
              <input
                id="gcu"
                type="checkbox"
                className="inputCheckbox"
                onChange={e => setGCUValidation(e.target.checked)}
                checked={GCUValidation}
                required
              />
              <div>
                <span>{t('dataShare.validCGU')}</span>
                <Button
                  size="small"
                  className="btnText"
                  onClick={() => setOpenCGUModal(true)}
                >
                  {t('dataShare.validCGU_button')}
                </Button>

                <span>{t('dataShare.validLegal')}</span>
                <Button
                  size="small"
                  className="btnText"
                  onClick={() => setOpenLegalNoticeModal(true)}
                >
                  {t('dataShare.validLegal_button')}
                </Button>
                <span>{t('dataShare.validLegal2')}</span>
              </div>
            </label>
          </div>
          <div className="terms-footer">
            <Button
              aria-label={t('dataShare.accessibility.button_accept')}
              onClick={handleTermValidate}
              disabled={!GCUValidation || !dataConsentValidation}
              className="btnPrimary"
            >
              {t('dataShare.button_accept')}
            </Button>
          </div>
        </>
      ) : (
        <>
          <div className="terms-content">
            <MinorUpdateContent />
          </div>
          <div className="terms-footer">
            <Button onClick={handleTermValidate} className="btnPrimary">
              {t('minorUpdate.button')}
            </Button>
          </div>
        </>
      )}

      <CGUModal
        open={openCGUModal}
        handleCloseClick={() => setOpenCGUModal(false)}
      />
      <LegalNoticeModal
        open={openLegalNoticeModal}
        handleCloseClick={() => setOpenLegalNoticeModal(false)}
      />
    </div>
  )
}

export default TermsView
